@@ -36,10 +36,15 @@ from kiwi.system.identifier import SystemIdentifier
 from kiwi.boot.image import BootImage
 from kiwi.storage.setup import DiskSetup
 from kiwi.storage.loop_device import LoopDevice
+from kiwi.storage.clone_device import CloneDevice
 from kiwi.firmware import FirmWare
 from kiwi.storage.disk import Disk
 from kiwi.storage.raid_device import RaidDevice
 from kiwi.storage.luks_device import LuksDevice
+from kiwi.storage.integrity_device import (
+    IntegrityDevice,
+    integrity_credentials_type
+)
 from kiwi.storage.device_provider import DeviceProvider
 from kiwi.filesystem import FileSystem
 from kiwi.filesystem.squashfs import FileSystemSquashFs
@@ -101,6 +106,10 @@ class DiskBuilder:
             xml_state.build_type.get_embed_verity_metadata()
         self.dosparttable_extended_layout = \
             xml_state.build_type.get_dosparttable_extended_layout()
+        self.boot_clone_count = int(xml_state.build_type.get_boot_clone()) \
+            if xml_state.build_type.get_boot_clone() else 0
+        self.root_clone_count = int(xml_state.build_type.get_root_clone()) \
+            if xml_state.build_type.get_root_clone() else 0
         self.custom_root_mount_args = xml_state.get_fs_mount_option_list()
         self.custom_root_creation_args = xml_state.get_fs_create_option_list()
         self.build_type_name = xml_state.get_build_type_name()
@@ -117,6 +126,12 @@ class DiskBuilder:
         self.hybrid_mbr = xml_state.build_type.get_gpt_hybrid_mbr()
         self.force_mbr = xml_state.build_type.get_force_mbr()
         self.luks = xml_state.get_luks_credentials()
+        self.integrity_root = xml_state.build_type.get_standalone_integrity()
+        self.integrity_keyfile = xml_state.build_type.get_integrity_keyfile()
+        self.integrity_key_description = \
+            xml_state.build_type.get_integrity_metadata_key_description()
+        self.root_filesystem_embed_integrity_metadata = \
+            xml_state.build_type.get_embed_integrity_metadata()
         self.luks_format_options = xml_state.get_luks_format_options()
         self.luks_os = xml_state.build_type.get_luksOS()
         self.xen_server = xml_state.is_xen_server()
@@ -225,7 +240,7 @@ class DiskBuilder:
 
         # a list of instances with the sync_data capability
         # representing the custom partitions area of the disk
-        system_custom_parts: List[FileSystemBase] = []
+        system_custom_parts: Dict[str, FileSystemBase] = {}
 
         if self.install_media and self.build_type_name != 'oem':
             raise KiwiInstallMediaError(
@@ -248,7 +263,9 @@ class DiskBuilder:
             self.boot_image.prepare()
 
         # precalculate needed disk size
-        disksize_mbytes = self.disk_setup.get_disksize_mbytes()
+        disksize_mbytes = self.disk_setup.get_disksize_mbytes(
+            root_clone=self.root_clone_count, boot_clone=self.boot_clone_count
+        )
 
         # create the disk
         log.info('Creating raw disk image %s', self.diskname)
@@ -299,6 +316,23 @@ class DiskBuilder:
             disk.public_partition_id_map['kiwi_RaidDev'] = \
                 device_map['root'].get_device()
 
+        # create integrity on current root device if requested
+        if self.integrity_root:
+            self.integrity_root = IntegrityDevice(
+                device_map['root'], defaults.INTEGRITY_ALGORITHM,
+                integrity_credentials_type(
+                    keydescription=self.integrity_key_description,
+                    keyfile=self.integrity_keyfile,
+                    keyfile_algorithm=defaults.INTEGRITY_KEY_ALGORITHM
+                )
+            )
+            self.integrity_root.create_dm_integrity()
+            device_map['integrity_root'] = device_map['root']
+            device_map['root'] = self.integrity_root.get_device()
+            if self.root_filesystem_is_overlay and \
+               self.root_filesystem_has_write_partition is False:
+                device_map['readonly'] = device_map['root']
+
         # create luks on current root device if requested
         luks_root = None
         if self.luks is not None:
@@ -404,9 +438,16 @@ class DiskBuilder:
                     self.root_dir + '/',
                     filesystem_custom_parameters
                 )
-                filesystem.create_on_device(
-                    label=self.disk_setup.get_root_label()
-                )
+                if self.root_filesystem_embed_integrity_metadata:
+                    filesystem.create_on_device(
+                        label=self.disk_setup.get_root_label(),
+                        size=-defaults.DM_METADATA_OFFSET,
+                        unit=defaults.UNIT.byte
+                    )
+                else:
+                    filesystem.create_on_device(
+                        label=self.disk_setup.get_root_label(),
+                    )
                 system = filesystem
 
         # create swap on current root device if requested
@@ -444,6 +485,8 @@ class DiskBuilder:
 
         self._write_crypttab_to_system_image(luks_root)
 
+        self._write_integritytab_to_system_image(self.integrity_root)
+
         self._write_generic_fstab_to_system_image(device_map, system)
 
         if self.initrd_system == 'dracut':
@@ -700,8 +743,8 @@ class DiskBuilder:
     def _build_custom_parts_filesystem(
         self, device_map: Dict,
         custom_partitions: Dict['str', ptable_entry_type]
-    ) -> List[FileSystemBase]:
-        filesystem_list = []
+    ) -> Dict[str, FileSystemBase]:
+        filesystem_dict = {}
         if custom_partitions:
             for map_name in sorted(custom_partitions.keys()):
                 if map_name in device_map:
@@ -715,8 +758,8 @@ class DiskBuilder:
                         filesystem.create_on_device(
                             label=map_name.upper()
                         )
-                        filesystem_list.append(filesystem)
-        return filesystem_list
+                        filesystem_dict[map_name] = filesystem
+        return filesystem_dict
 
     def _build_spare_filesystem(self, device_map: Dict) -> Optional[FileSystemBase]:
         if 'spare' in device_map and self.spare_part_fs:
@@ -809,12 +852,18 @@ class DiskBuilder:
             disksize_used_mbytes += partition_mbsize
 
         if self.disk_setup.need_boot_partition():
-            log.info('--> creating boot partition')
+            log.info(
+                '--> creating boot partition [with {0} clone(s)]'.format(
+                    self.boot_clone_count
+                )
+            )
             partition_mbsize = self.disk_setup.boot_partition_size()
             disk.create_boot_partition(
-                partition_mbsize
+                partition_mbsize, self.boot_clone_count
             )
-            disksize_used_mbytes += partition_mbsize
+            disksize_used_mbytes += \
+                (self.boot_clone_count + 1) * partition_mbsize if \
+                self.boot_clone_count else partition_mbsize
 
         if self.swap_mbytes:
             if not self.volume_manager_name or self.volume_manager_name != 'lvm':
@@ -858,9 +907,11 @@ class DiskBuilder:
                     os.path.getsize(squashed_root_file.name) / 1048576
                 ) + Defaults.get_min_partition_mbytes()
             disk.create_root_readonly_partition(
-                squashed_rootfs_mbsize
+                squashed_rootfs_mbsize, self.root_clone_count
             )
-            disksize_used_mbytes += squashed_rootfs_mbsize
+            disksize_used_mbytes += \
+                (self.root_clone_count + 1) * squashed_rootfs_mbsize if \
+                self.root_clone_count else squashed_rootfs_mbsize
 
         if self.spare_part_mbsize and self.spare_part_is_last:
             rootfs_mbsize = disksize_mbytes - disksize_used_mbytes - \
@@ -874,15 +925,39 @@ class DiskBuilder:
                 '--> overlayroot explicitly requested no write partition'
             )
         else:
+            root_clone_count = self.root_clone_count
+            if self.root_filesystem_is_overlay:
+                # in overlay mode an eventual root clone is created from
+                # the root readonly partition and not from the root (rw)
+                # partition. Thus no further action needed here in this
+                # case
+                root_clone_count = 0
+            if root_clone_count:
+                clone_rootfs_mbsize = int(
+                    (disksize_mbytes - disksize_used_mbytes) / (root_clone_count + 1)
+                ) + Defaults.get_min_partition_mbytes()
+                rootfs_mbsize = f'clone:all_free:{clone_rootfs_mbsize}'
             if self.volume_manager_name and self.volume_manager_name == 'lvm':
-                log.info('--> creating LVM root partition')
-                disk.create_root_lvm_partition(rootfs_mbsize)
+                log.info(
+                    '--> creating {0} partition [with {1} clone(s)]'.format(
+                        'root(LVM)', root_clone_count
+                    )
+                )
+                disk.create_root_lvm_partition(rootfs_mbsize, root_clone_count)
             elif self.mdraid:
-                log.info('--> creating mdraid root partition')
-                disk.create_root_raid_partition(rootfs_mbsize)
+                log.info(
+                    '--> creating {0} partition [with {1} clone(s)]'.format(
+                        f'root(mdraid={self.mdraid})', root_clone_count
+                    )
+                )
+                disk.create_root_raid_partition(rootfs_mbsize, root_clone_count)
             else:
-                log.info('--> creating root partition')
-                disk.create_root_partition(rootfs_mbsize)
+                log.info(
+                    '--> creating root partition [with {0} clone(s)]'.format(
+                        root_clone_count
+                    )
+                )
+                disk.create_root_partition(rootfs_mbsize, root_clone_count)
 
         if self.spare_part_mbsize and self.spare_part_is_last:
             log.info('--> creating spare partition')
@@ -908,7 +983,11 @@ class DiskBuilder:
 
         disk.map_partitions()
 
-        return disk.get_device()
+        device_map = disk.get_device()
+        device_map['origin_root'] = \
+            device_map.get('readonly') or device_map['root']
+
+        return device_map
 
     def _write_partition_id_config_to_boot_image(self, disk: Disk) -> None:
         log.info('Creating config.partids in boot system')
@@ -938,6 +1017,19 @@ class DiskBuilder:
                 os.sep + os.sep.join(['etc', os.path.basename(filename)])
             )
 
+    def _write_integritytab_to_system_image(
+        self, integrity_root: Optional[IntegrityDevice]
+    ) -> None:
+        if integrity_root:
+            log.info('Creating etc/integritytab')
+            filename = ''.join(
+                [self.root_dir, '/etc/integritytab']
+            )
+            integrity_root.create_integritytab(filename)
+            self.boot_image.include_file(
+                os.sep + os.sep.join(['etc', os.path.basename(filename)])
+            )
+
     def _write_crypttab_to_system_image(
         self, luks_root: Optional[LuksDevice]
     ) -> None:
@@ -1151,16 +1243,27 @@ class DiskBuilder:
         system_boot: Optional[FileSystemBase],
         system_efi: Optional[FileSystemBase],
         system_spare: Optional[FileSystemBase],
-        system_custom_parts: List[FileSystemBase]
+        system_custom_parts: Dict[str, FileSystemBase]
     ) -> None:
         log.info('Syncing system to image')
         if system_spare:
             log.info('--> Syncing spare partition data')
             system_spare.sync_data()
 
-        for system_custom_part in system_custom_parts:
+        for map_name in sorted(system_custom_parts.keys()):
+            system_custom_part = system_custom_parts[map_name]
             log.info('--> Syncing custom partition(s) data')
             system_custom_part.sync_data()
+            if device_map.get(f'{map_name}clone1'):
+                log.info(
+                    f'--> Dumping {map_name} clone data at extra partition'
+                )
+                system_custom_part_clone = CloneDevice(
+                    system_custom_part.device_provider, self.root_dir
+                )
+                system_custom_part_clone.clone(
+                    self._get_clone_devices(f'{map_name}clone', device_map)
+                )
 
         if system_efi:
             log.info('--> Syncing EFI boot data to EFI partition')
@@ -1171,6 +1274,13 @@ class DiskBuilder:
             system_boot.sync_data(
                 self._get_exclude_list_for_boot_data_sync()
             )
+            if device_map.get('bootclone1'):
+                log.info(
+                    '--> Dumping boot clone data at extra partition'
+                )
+                CloneDevice(system_boot.device_provider, self.root_dir).clone(
+                    self._get_clone_devices('bootclone', device_map)
+                )
 
         log.info('--> Syncing root filesystem data')
         if self.root_filesystem_is_overlay:
@@ -1221,6 +1331,13 @@ class DiskBuilder:
                 squashed_root.create_verification_metadata(
                     readonly_target
                 )
+            if device_map.get('rootclone1'):
+                log.info(
+                    '--> Dumping readonly root clone data at extra partition'
+                )
+                CloneDevice(device_map['origin_root'], self.root_dir).clone(
+                    self._get_clone_devices('rootclone', device_map)
+                )
         elif self.root_filesystem_verity_blocks:
             root_target = device_map['root'].get_device()
             root_target_bytesize = device_map['root'].get_byte_size(
@@ -1232,7 +1349,7 @@ class DiskBuilder:
                 self.root_filesystem_verity_blocks != 'all' else None
             ).get_hash_byte_size()
             if self.root_filesystem_embed_verity_metadata:
-                verity_root_file_bytes -= defaults.VERIFICATION_METADATA_OFFSET
+                verity_root_file_bytes -= defaults.DM_METADATA_OFFSET
             verity_root_file = Temporary().new_file()
             loop_provider = LoopDevice(
                 verity_root_file.name,
@@ -1279,10 +1396,32 @@ class DiskBuilder:
                 filesystem.create_verification_metadata(
                     root_target
                 )
+            if device_map.get('rootclone1'):
+                log.info(
+                    '--> Dumping root clone data at extra partition'
+                )
+                CloneDevice(device_map['origin_root'], self.root_dir).clone(
+                    self._get_clone_devices('rootclone', device_map)
+                )
         else:
             system.sync_data(
                 self._get_exclude_list_for_root_data_sync(device_map)
             )
+            if device_map.get('rootclone1'):
+                log.info(
+                    '--> Dumping root clone data at extra partition'
+                )
+                CloneDevice(device_map['origin_root'], self.root_dir).clone(
+                    self._get_clone_devices('rootclone', device_map)
+                )
+
+        if self.integrity_root and \
+           self.root_filesystem_embed_integrity_metadata:
+            log.info('--> Creating integrity metadata...')
+            self.integrity_root.create_integrity_metadata()
+            log.info('--> Signing integrity metadata...')
+            self.integrity_root.sign_integrity_metadata()
+            self.integrity_root.write_integrity_metadata()
 
     def _install_bootloader(
         self, device_map: Dict, disk, system: Any
@@ -1322,14 +1461,15 @@ class DiskBuilder:
 
         if self.bootloader != 'custom':
             # create bootloader config prior bootloader installation
-            self.bootloader_config.setup_disk_image_config(
-                boot_options=custom_install_arguments
-            )
-            if 's390' in self.arch:
-                self.bootloader_config.write()
-
-            # cleanup bootloader config resources taken prior to next steps
-            del self.bootloader_config
+            try:
+                self.bootloader_config.setup_disk_image_config(
+                    boot_options=custom_install_arguments
+                )
+                if 's390' in self.arch:
+                    self.bootloader_config.write()
+            finally:
+                # cleanup bootloader config resources taken prior to next steps
+                del self.bootloader_config
 
             log.debug(
                 "custom arguments for bootloader installation %s",
@@ -1392,3 +1532,12 @@ class DiskBuilder:
                     self.root_dir + ''.join(['/boot/', boot_names.initrd_name])
                 ]
             )
+
+    def _get_clone_devices(
+        self, match: str, device_map: Dict[str, DeviceProvider]
+    ) -> List[DeviceProvider]:
+        result = []
+        for map_name in sorted(device_map.keys()):
+            if map_name.startswith(match):
+                result.append(device_map[map_name])
+        return result
